/* See LICENSE file for copyright and license details. */

#include <X11/XF86keysym.h>

/* appearance */
static const char font[]            = "-*-terminus-medium-r-*-*-12-*-*-*-*-*-*-r";
//static const char font[] = "Cousine:style=Bold:size=9";
//static const char font[] = "-*-*-medium-*-*-*-14-*-*-*-*-*-iso10646-*";
//static const char font[] = "-*-xbmicons-medium-r-*-*-12-*-*-*-*-*-*-*" ","
//                           "-*-terminus-medium-r-*-*-12-*-*-*-*-*-*-*";

static const char normbordercolor[] = "#333333";
static const char normbgcolor[]     = "#333333";
static const char normfgcolor[]     = "#00F0FF";
static const char selbordercolor[]  = "#00C0CC";
static const char selbgcolor[]      = "#00F0FF";
static const char selfgcolor[]      = "#222222";
static const char floatnormbordercolor[] = "#503500";
static const char floatselbordercolor[]  = "#E08500";
static const char urgbordercolor[]  = "#FF0000";
//static const char masternormbordercolor[] = "#005500";
static const char masternormbordercolor[] = "#004044";
static const char masterselbordercolor[]  = "#00F0FF";
static const unsigned int borderpx  = 4;        /* border pixel of windows */
static const unsigned int gappx     = 2;        /* gap pixel between windows */
static const unsigned int snap      = 13;       /* snap pixel */
static const Bool showbar           = True;     /* False means no bar */
static const Bool topbar            = True;     /* False means bottom bar */
static const Bool extrabar          = True;     /* False means no extra bar */
//static Bool useicons          = True;     // True means use xbm icons if toolbar symbols
static Bool useicons          = False;     // False means use ascii in toolbar symbols

/* tagging */
static const char *tags[] = { "1", "2", "3", "4", "5", "6", "7", "8", "9" };

static const Rule rules[] = {
	/* class      instance    title       tags mask     iscentered   isfloating   monitor  opacity*/
	{ "xcalc",    NULL,       NULL,       0,            False,       True,        -1,      0.85 },
	{ "Gimp",     NULL,       NULL,       0,            False,       True,        -1,      0.85 },
	{ "Firefox",  NULL,       NULL,       1 << 8,       False,       False,       -1,      0.85 },
};

/* layout(s) */
static const float mfact      = 0.55; /* factor of master area size [0.05..0.95] */
static const int nmaster      = 1;    /* number of clients in master area */
static const Bool resizehints = True; /* True means respect size hints in tiled resizals */
static const double defaultopacity  = 0.75;
static const double shade     = 0.5;      /* opacity of unfocussed clients */

#include "focusmaster.c"
#include "push.c"
#include "tilemovemouse.c"
#include "lay.bstack.c"
#include "lay.grid.c"
#include "lay.gaplessgrid.c"
#include "lay.tcl.c"
#include "lay.fibonacci.c"


static const Layout layouts[] = {
	/* symbol     arrange function */
	{ "[]=",      tile },    /* first entry is default */
	{ "TTT",      bstack },  /* bottom stack layout */
	{ "><>",      NULL },    /* no layout function means floating behavior */
	{ "###",      grid },    /* grid layout */
	{ "+++",      gaplessgrid }, /* gapless grid layout */
	{ "|||",      tcl },     /* master is centered */
	{ "[@]",      spiral },  /* golden fibonacci spiral layout */
	{ "[\\]",     dwindle },/* reverse golden fibonacci spiral layout */
	{ "[M]",      monocle }, /* monocle layout */
	{ .symbol = NULL,   .arrange = NULL    },  /* last entry is empty terminator for layout cycling */
};


/* key definitions */
#define MODWKEY          Mod4Mask
#define MODKEY           Mod1Mask
#define TAGKEYS(KEY,TAG) \
	{ MODKEY,                       KEY,      view,           {.ui = 1 << TAG} }, \
	{ MODKEY|ControlMask,           KEY,      toggleview,     {.ui = 1 << TAG} }, \
	{ MODKEY|ShiftMask,             KEY,      tag,            {.ui = 1 << TAG} }, \
	{ MODKEY|ControlMask|ShiftMask, KEY,      toggletag,      {.ui = 1 << TAG} },

/* helper for spawning shell commands in the pre dwm-5.0 fashion */
#define SHCMD(cmd) { .v = (const char*[]){ "/bin/sh", "-c", cmd, NULL } }

/* commands */
static const char *dmenucmd[] = { "dmenu_run", "-fn", font, "-nb", normbgcolor, "-nf", normfgcolor, "-sb", selbgcolor, "-sf", selfgcolor, NULL };
//static const char *dmenu_find[]  = { "dmenu_finder", NULL };
//static const char *dmenu_fm[]    = { "dmenu_fm", NULL };
//static const char *dmenu_net[]   = { "dmenu_netctl", NULL };
//static const char *dmenu_halt[]  = { "dmenu_shutdown", NULL };


static const char *kbdcmd[]   = { "svkbd-call.sh", NULL };
static const char *termcmd[]  = { "uxterm", NULL };


static const char *xkill[]    = { "xkill", NULL };
static const char *xlock[]    = { "slock", NULL };

static const char *disp_off[]  = { "xset", "dpms", "force", "off", NULL };
static const char *bl_up[]     = { "xbacklight", "-inc", "20%", NULL };
static const char *bl_down[]   = { "xbacklight", "-dec", "20%", NULL };


// ...involving moc player //t
static const char *play[]    = { "mocp", "-G", NULL }; //toggle play/pause
static const char *next[]    = { "mocp", "-f", NULL }; //next track
static const char *prev[]    = { "mocp", "-r", NULL }; //prev track
static const char *stop[]    = { "mocp", "-s", NULL }; //stop playing


static const char *vol_mute[] = { "amixer", "-q", "set", "PCM", "toggle", NULL };
static const char *vol_up[]   = { "amixer", "-q", "set", "PCM", "5%+", "unmute", NULL };
static const char *vol_down[] = { "amixer", "-q", "set", "PCM", "5%-", "unmute", NULL };

static const char *scrot[]    = { "scrot", "/tmp/screenshot-%d-%m-%Y-%R.png", NULL };

static const char *web_ff[]   = { "firefox", NULL, "firefox" };
static const char *web_ch[]   = { "chromium", NULL, "chromium" };

#include "selfrestart.c"

static Key keys[] = {
	/* modifier                     key        function        argument */
///////////////////////////////////////////////////////////////////////////////////////////
/* spawn da beasts */
//	{ MODKEY,                       XK_k,      spawn,          {.v = kbdcmd } }, /// inactual for desktop-mode and useless for touch screen-mode - only for testing yet //t
	{ MODKEY,                       XK_p,      spawn,          {.v = dmenucmd } },
	{ MODKEY|ShiftMask,             XK_Return, spawn,          {.v = termcmd } },
	{ MODWKEY|ShiftMask,            XK_Return, spawn,          {.v = termcmd } },

/* bar control */
	{ MODKEY,                       XK_b,      togglebar,      {0} },
	{ MODKEY,                       XK_b,      toggleextrabar, {0} }, // both bars are switching on/off with the same key-combo
//	{ MODWKEY,                      XK_b,      toggleextrabar, {0} }, // down bar controlled by MOD4+b
//	{ MODKEY|ShiftMask,             XK_b,      toggleextrabar, {0} }, // down bar controlled by MOD1+SHIFT+b

/* browsers spawn */
	{ MODWKEY,                      XK_F1,      runorraise,    {.v = web_ff } },
	{ MODWKEY,                      XK_F2,      runorraise,    {.v = web_ch } },

/* volume control spawn */
	{ MODWKEY,           XK_F10,                     spawn,          {.v = vol_mute } },
	{ MODWKEY,           XK_F11,                     spawn,          {.v = vol_down } },
	{ MODWKEY,           XK_F12,                     spawn,          {.v = vol_up } },
	{ 0,                 XF86XK_AudioRaiseVolume,    spawn,          {.v = vol_up } },
	{ 0,                 XF86XK_AudioLowerVolume,    spawn,          {.v = vol_down } },
	{ 0,                 XF86XK_AudioMute,           spawn,          {.v = vol_mute } },
/* MOC player control spawn */
	{ 0,                 XF86XK_AudioPlay,           spawn,          {.v = play } },
	{ 0,                 XF86XK_AudioNext,           spawn,          {.v = next } },
	{ 0,                 XF86XK_AudioPrev,           spawn,          {.v = prev } },
	{ 0,                 XF86XK_AudioStop,           spawn,          {.v = stop } },
/* display control spawn */
	{ 0,                 XF86XK_Display,             spawn,          {.v = disp_off } },
	{ MODWKEY,           XK_F5,                      spawn,          {.v = bl_down } },
	{ MODWKEY,           XK_F6,                      spawn,          {.v = bl_up } },
/* opacity settings */
	{ MODWKEY,           XK_equal,                   spawn,          SHCMD("transset -a --inc .1") },
	{ MODWKEY,           XK_minus,                   spawn,          SHCMD("transset -a --dec .1") },
/* screenee-shot! */
	{ 0,                 XK_Print,                   spawn,          {.v = scrot } },

/* other usable spawns */
	{ MODWKEY,           XK_x,                       spawn,          {.v = xkill } },
	{ MODWKEY,           XK_l,                       spawn,          {.v = xlock } },
//	{ MODWKEY,           XK_s,                       spawn,          {.v = dmenu_find } },
//	{ MODWKEY,           XK_o,                       spawn,          {.v = dmenu_fm } },
//	{ MODWKEY,           XK_n,                       spawn,          {.v = dmenu_net } },
/* careful! halt! */
//	{ MODWKEY,           XK_Escape,                  spawn,          {.v = dmenu_halt } },

/* move focus through the stack */
	{ MODKEY,                       XK_comma,        focusstack,     {.i = -1 } },
	{ MODKEY,                       XK_period,       focusstack,     {.i = +1 } },
	{ MODKEY,                       XK_j,            focusstack,     {.i = +1 } },
	{ MODKEY,                       XK_k,            focusstack,     {.i = -1 } },

/* switch stack view: number of master-zone windows management */
	{ MODKEY,                       XK_i,            incnmaster,     {.i = +1 } },
	{ MODKEY,                       XK_d,            incnmaster,     {.i = -1 } },

/* resize horizontal master/stack */
	{ MODKEY|ShiftMask,             XK_equal,        setmfact,    {.f = +0.05} },
	{ MODKEY|ShiftMask,             XK_minus,        setmfact,    {.f = -0.05} },
	{ MODKEY|ShiftMask,             XK_backslash,    setmfact,    {.f =  0.00} },
	{ MODKEY,                       XK_h,            setmfact,    {.f = -0.05} },
	{ MODKEY,                       XK_l,            setmfact,    {.f = +0.05} },
	{ MODKEY,                       XK_o,            setmfact,    {.f =  0.00} },

/* resize vertical master/stack */
	{ MODKEY,                       XK_equal,        setcfact,    {.f = +0.25} },
	{ MODKEY,                       XK_minus,        setcfact,    {.f = -0.25} },
	{ MODKEY,                       XK_backslash,    setcfact,    {.f =  0.00} },
	{ MODKEY|ShiftMask,             XK_h,            setcfact,    {.f = +0.25} },
	{ MODKEY|ShiftMask,             XK_l,            setcfact,    {.f = -0.25} },
	{ MODKEY|ShiftMask,             XK_o,            setcfact,    {.f =  0.00} },

/* focus master window */
	{ MODKEY|ControlMask,           XK_Return,       focusmaster,    {0} },

/* push the client up/down */
	{ MODKEY|ShiftMask,             XK_j,            pushdown,    {.i = +1 } },
	{ MODKEY|ShiftMask,             XK_k,            pushup,      {.i = -1 } },

////////////////////////////////////////////////////////////////////////////////////////////////////
// one-left-hand-control
// one-left-hand: spawn the beasts
	{ MODKEY,                       XK_q, spawn,          {.v = termcmd } },
	{ MODKEY,                       XK_e, spawn,          {.v = dmenucmd } },
// one-left-hand: switch active client
	{ MODKEY,                       XK_z, zoom,           {0} },
	{ MODKEY,                       XK_w, focusstack,     {.i = -1 } },
	{ MODKEY,                       XK_s, focusstack,     {.i = +1 } },
// one-left-hand: push the client up/down
	{ MODKEY|ShiftMask,             XK_q, pushup,          {.i = -1 } },
	{ MODKEY|ShiftMask,             XK_z, pushdown,        {.i = +1 } },
// one-left-hand: resize vertical master/stack
	{ MODKEY|ShiftMask,             XK_w, setcfact,       {.f = +0.25} },
	{ MODKEY|ShiftMask,             XK_s, setcfact,       {.f = -0.25} },
	{ MODKEY|ShiftMask,             XK_x, setcfact,       {.f =  0.00} },
// one-left-hand: resize horizontal master/stack
	{ MODKEY|ShiftMask,             XK_a, setmfact,       {.f = -0.05} },
	{ MODKEY|ShiftMask,             XK_d, setmfact,       {.f = +0.05} },
// focus master /home
	{ MODKEY|ShiftMask,             XK_h, focusmaster,    {0} },
//
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// one-left-hand-control, ex-windows key mod, aka MOD4
// one-left-hand: spawn the beasts
	{ MODWKEY,                       XK_q, spawn,          {.v = termcmd } },
	{ MODWKEY,                       XK_e, spawn,          {.v = dmenucmd } },
// one-left-hand: switch active client
	{ MODWKEY,                       XK_z, zoom,           {0} },
	{ MODWKEY,                       XK_w, focusstack,     {.i = -1 } },
	{ MODWKEY,                       XK_s, focusstack,     {.i = +1 } },
// one-left-hand: push the client up/down
	{ MODWKEY|ShiftMask,             XK_q, pushup,          {.i = -1 } },
	{ MODWKEY|ShiftMask,             XK_z, pushdown,        {.i = +1 } },
// one-left-hand: resize vertical master/stack
	{ MODWKEY|ShiftMask,             XK_w, setcfact,       {.f = +0.25} },
	{ MODWKEY|ShiftMask,             XK_s, setcfact,       {.f = -0.25} },
	{ MODWKEY|ShiftMask,             XK_x, setcfact,       {.f =  0.00} },
// one-left-hand: resize horizontal master/stack
	{ MODWKEY|ShiftMask,             XK_a, setmfact,       {.f = -0.05} },
	{ MODWKEY|ShiftMask,             XK_d, setmfact,       {.f = +0.05} },
// focus master /home
	{ MODWKEY|ShiftMask,             XK_h, focusmaster,    {0} },
//
////////////////////////////////////////////////////////////////////////////////////////////////////

	{ MODKEY,                       XK_Return, zoom,           {0} },
	{ MODKEY,                       XK_Tab,    view,           {0} },
	{ NULL,                         XK_F17,    killclient,     {0} },
	{ MODKEY|ShiftMask,             XK_c,      killclient,     {0} },
/* layouts control */
	{ MODKEY,                       XK_t,      setlayout,      {.v = &layouts[0]} }, // tiled
	{ MODKEY,                       XK_y,      setlayout,      {.v = &layouts[1]} }, // bottom stack
	{ MODKEY,                       XK_f,      setlayout,      {.v = &layouts[2]} }, // floating
	{ MODKEY,                       XK_g,      setlayout,      {.v = &layouts[3]} }, // grid
	{ MODKEY,                       XK_m,      setlayout,      {.v = &layouts[8]} }, // monocle
	{ MODKEY|ControlMask,           XK_space,  prevlayout,     {0} },
	{ MODKEY,                       XK_space,  nextlayout,     {0} },
/*---------------*/
	{ MODKEY|ShiftMask,             XK_space,  togglefloating, {0} },
	{ MODKEY,                       XK_0,      view,           {.ui = ~0 } },
	{ MODKEY|ShiftMask,             XK_0,      tag,            {.ui = ~0 } },
	{ MODKEY,                       XK_bracketleft,  focusmon, {.i = -1 } },
	{ MODKEY,                       XK_bracketright, focusmon, {.i = +1 } },
	{ MODKEY|ShiftMask,             XK_comma,  tagmon,         {.i = -1 } },
	{ MODKEY|ShiftMask,             XK_period, tagmon,         {.i = +1 } },
	TAGKEYS(                        XK_1,                      0)
	TAGKEYS(                        XK_2,                      1)
	TAGKEYS(                        XK_3,                      2)
	TAGKEYS(                        XK_4,                      3)
	TAGKEYS(                        XK_5,                      4)
	TAGKEYS(                        XK_6,                      5)
	TAGKEYS(                        XK_7,                      6)
	TAGKEYS(                        XK_8,                      7)
	TAGKEYS(                        XK_9,                      8)
	{ MODKEY|ControlMask,           XK_r,      self_restart,   {0} },
	{ MODKEY|ControlMask,           XK_q,      quit,           {0} },
};

/* button definitions */
/* click can be ClkLtSymbol, ClkStatusText, ClkWinTitle, ClkClientWin, or ClkRootWin */
static Button buttons[] = {
	/* click                event mask      button          function        argument */
	{ ClkLtSymbol,          0,              Button1,        nextlayout,      {0} },
	{ ClkLtSymbol,          0,              Button3,        setlayout,      {.v = &layouts[2]} },
	{ ClkWinTitle,          0,              Button2,        zoom,           {0} },
	{ ClkWinTitle,          0,              Button2,        spawn,          {.v = dmenucmd } },
	{ ClkWinTitle,          0,              Button1,        spawn,          {.v = termcmd } },
	{ ClkStatusText,        0,              Button1,        spawn,          {.v = kbdcmd } },
	{ ClkStatusText,        0,              Button3,        spawn,          {.v = termcmd } },
	{ ClkStatusText,        0,              Button4,        spawn,          {.v = vol_up } },
	{ ClkStatusText,        0,              Button5,        spawn,          {.v = vol_down } },
//	{ ClkClientWin,         MODKEY,         Button1,        movemouse,      {0} }, // 1b for client temporary off
	{ ClkClientWin,         MODKEY,         Button2,        movemouse,      {0} }, // 1b replaced by the 2b for movemouse
//	{ ClkClientWin,         MODKEY,         Button2,        togglefloating, {0} }, // 2b for client temporary off
	{ ClkClientWin,         MODKEY,         Button3,        resizemouse,    {0} }, // 2b replaced by the 3b for resize
	{ ClkClientWin,         MODKEY|ShiftMask,   Button1,    resizemouse,    {0} },
	{ ClkTagBar,            0,              Button1,        view,           {0} },
	{ ClkTagBar,            0,              Button3,        toggleview,     {0} },
	{ ClkTagBar,            MODKEY,         Button1,        tag,            {0} },
	{ ClkTagBar,            MODKEY,         Button3,        toggletag,      {0} },
	{ ClkStatusText,        MODKEY,         Button4,        spawn,          {.v = bl_up } },
	{ ClkStatusText,        MODKEY,         Button5,        spawn,          {.v = bl_down } },
// experimental: move tiles with the mouse //t
	{ ClkClientWin,         MODKEY,         Button1,        tilemovemouse,  {0} },
//	{ ClkClientWin,         0,              Button1,        tilemovemouse,  {0} }, /// testing only case fot one-click-drag tile
};

/* toolbar definition */
// text-mode version  of toolbar
static Tool tools[] = {
	// symbol     function        argument 
	{ "[>x<]",    killclient,     {0} },
	{ "[ZOOM]",   zoom,         {0} },
	{ "<M>",      setlayout,    {.v = &layouts[8]} },
	{ "[^]",      pushup,       {.i = -1} },
	{ "[v]",      pushdown,     {.i = +1} },
	{ "[--]",     setmfact,     {.f = -0.05} },
	{ "[++]",     setmfact,     {.f = +0.05} },
	{ "[KBD]",    spawn,        {.v = kbdcmd } },
	{ "[>_]",     spawn,        {.v = termcmd } },
	{ "[MENU]",   spawn,        {.v = dmenucmd } },
	{ "[QUIT]",   quit,         {0} },
};

/*
// icon-mode version  of toolbar
static Tool tools[] = {
	// symbol     function        argument
	{ "/usr/local/share/dwm/icons/dwm.win.close.xbm",   killclient,   {0} },
	{ "/usr/local/share/dwm/icons/dwm.win.zoom.xbm",    zoom,         {0} },
	{ "/usr/local/share/dwm/icons/dwm.main.dec.xbm",    setmfact,     {.f = -0.05} },
	{ "/usr/local/share/dwm/icons/dwm.main.inc.xbm",    setmfact,     {.f = +0.05} },
	{ "/usr/local/share/dwm/icons/dwm.stack.up.xbm",    pushup,       {.i = -1} },
	{ "/usr/local/share/dwm/icons/dwm.stack.down.xbm",  pushdown,     {.i = +1} },
	{ "/usr/local/share/dwm/icons/dwm.term.new.xbm",    spawn,        {.v = termcmd } },
	{ "/usr/local/share/dwm/icons/dwm.svkbd.xbm",       spawn,        {.v = kbdcmd } },
};
*/
